/*
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import { Component, OnInit, Input } from '@angular/core';
import { FormGroup, FormControl, Validators, FormBuilder } from '@angular/forms';
import { ZonaService } from './Zona.service';
import 'rxjs/add/operator/toPromise';

@Component({
  selector: 'app-zona',
  templateUrl: './Zona.component.html',
  styleUrls: ['./Zona.component.css'],
  providers: [ZonaService]
})
export class ZonaComponent implements OnInit {

  myForm: FormGroup;

  private allAssets;
  private asset;
  private currentId;
  private errorMessage;

  zonaId = new FormControl('', Validators.required);
  ownershipType = new FormControl('', Validators.required);
  suites = new FormControl('', Validators.required);
  dobles = new FormControl('', Validators.required);
  individuales = new FormControl('', Validators.required);
  nickName = new FormControl('', Validators.required);

  constructor(public serviceZona: ZonaService, fb: FormBuilder) {
    this.myForm = fb.group({
      zonaId: this.zonaId,
      ownershipType: this.ownershipType,
      suites: this.suites,
      dobles: this.dobles,
      individuales: this.individuales,
      nickName: this.nickName
    });
  };

  ngOnInit(): void {
    this.loadAll();
  }

  loadAll(): Promise<any> {
    const tempList = [];
    return this.serviceZona.getAll()
    .toPromise()
    .then((result) => {
      this.errorMessage = null;
      result.forEach(asset => {
        tempList.push(asset);
      });
      this.allAssets = tempList;
    })
    .catch((error) => {
      if (error === 'Server error') {
        this.errorMessage = 'Could not connect to REST server. Please check your configuration details';
      } else if (error === '404 - Not Found') {
        this.errorMessage = '404 - Could not find API route. Please check your available APIs.';
      } else {
        this.errorMessage = error;
      }
    });
  }

	/**
   * Event handler for changing the checked state of a checkbox (handles array enumeration values)
   * @param {String} name - the name of the asset field to update
   * @param {any} value - the enumeration value for which to toggle the checked state
   */
  changeArrayValue(name: string, value: any): void {
    const index = this[name].value.indexOf(value);
    if (index === -1) {
      this[name].value.push(value);
    } else {
      this[name].value.splice(index, 1);
    }
  }

	/**
	 * Checkbox helper, determining whether an enumeration value should be selected or not (for array enumeration values
   * only). This is used for checkboxes in the asset updateDialog.
   * @param {String} name - the name of the asset field to check
   * @param {any} value - the enumeration value to check for
   * @return {Boolean} whether the specified asset field contains the provided value
   */
  hasArrayValue(name: string, value: any): boolean {
    return this[name].value.indexOf(value) !== -1;
  }

  addAsset(form: any): Promise<any> {
    this.asset = {
      $class: 'org.hotelandia.zona.Zona',
      'zonaId': this.zonaId.value,
      'ownershipType': this.ownershipType.value,
      'suites': this.suites.value,
      'dobles': this.dobles.value,
      'individuales': this.individuales.value,
      'nickName': this.nickName.value
    };

    this.myForm.setValue({
      'zonaId': null,
      'ownershipType': null,
      'suites': null,
      'dobles': null,
      'individuales': null,
      'nickName': null
    });

    return this.serviceZona.addAsset(this.asset)
    .toPromise()
    .then(() => {
      this.errorMessage = null;
      this.myForm.setValue({
        'zonaId': null,
        'ownershipType': null,
        'suites': null,
        'dobles': null,
        'individuales': null,
        'nickName': null
      });
      this.loadAll();
    })
    .catch((error) => {
      if (error === 'Server error') {
          this.errorMessage = 'Could not connect to REST server. Please check your configuration details';
      } else {
          this.errorMessage = error;
      }
    });
  }


  updateAsset(form: any): Promise<any> {
    this.asset = {
      $class: 'org.hotelandia.zona.Zona',
      'ownershipType': this.ownershipType.value,
      'suites': this.suites.value,
      'dobles': this.dobles.value,
      'individuales': this.individuales.value,
      'nickName': this.nickName.value
    };

    return this.serviceZona.updateAsset(form.get('zonaId').value, this.asset)
    .toPromise()
    .then(() => {
      this.errorMessage = null;
      this.loadAll();
    })
    .catch((error) => {
      if (error === 'Server error') {
        this.errorMessage = 'Could not connect to REST server. Please check your configuration details';
      } else if (error === '404 - Not Found') {
        this.errorMessage = '404 - Could not find API route. Please check your available APIs.';
      } else {
        this.errorMessage = error;
      }
    });
  }


  deleteAsset(): Promise<any> {

    return this.serviceZona.deleteAsset(this.currentId)
    .toPromise()
    .then(() => {
      this.errorMessage = null;
      this.loadAll();
    })
    .catch((error) => {
      if (error === 'Server error') {
        this.errorMessage = 'Could not connect to REST server. Please check your configuration details';
      } else if (error === '404 - Not Found') {
        this.errorMessage = '404 - Could not find API route. Please check your available APIs.';
      } else {
        this.errorMessage = error;
      }
    });
  }

  setId(id: any): void {
    this.currentId = id;
  }

  getForm(id: any): Promise<any> {

    return this.serviceZona.getAsset(id)
    .toPromise()
    .then((result) => {
      this.errorMessage = null;
      const formObject = {
        'zonaId': null,
        'ownershipType': null,
        'suites': null,
        'dobles': null,
        'individuales': null,
        'nickName': null
      };

      if (result.zonaId) {
        formObject.zonaId = result.zonaId;
      } else {
        formObject.zonaId = null;
      }

      if (result.ownershipType) {
        formObject.ownershipType = result.ownershipType;
      } else {
        formObject.ownershipType = null;
      }

      if (result.suites) {
        formObject.suites = result.suites;
      } else {
        formObject.suites = null;
      }

      if (result.dobles) {
        formObject.dobles = result.dobles;
      } else {
        formObject.dobles = null;
      }

      if (result.individuales) {
        formObject.individuales = result.individuales;
      } else {
        formObject.individuales = null;
      }

      if (result.nickName) {
        formObject.nickName = result.nickName;
      } else {
        formObject.nickName = null;
      }

      this.myForm.setValue(formObject);

    })
    .catch((error) => {
      if (error === 'Server error') {
        this.errorMessage = 'Could not connect to REST server. Please check your configuration details';
      } else if (error === '404 - Not Found') {
        this.errorMessage = '404 - Could not find API route. Please check your available APIs.';
      } else {
        this.errorMessage = error;
      }
    });
  }

  resetForm(): void {
    this.myForm.setValue({
      'zonaId': null,
      'ownershipType': null,
      'suites': null,
      'dobles': null,
      'individuales': null,
      'nickName': null
      });
  }

}
